@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.49 2000/01/26 22:10:15 nisse Exp $
+ * $Id: channel.h,v 1.52 2000/02/06 21:43:41 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -32,6 +32,7 @@
 #include "connection.h"
 #include "parse.h"
 #include "server_pty.h"
+#include "write_buffer.h"
 
 #define GABA_DECLARE
 #include "channel.h.x"
@@ -67,7 +68,13 @@
        ; is needed for deallocating it.
        ;; (local_number . UINT32)
        
-       ; Where to pass errors
+       ; Where to pass errors. This is used for two different
+       ; purposes: If opening the channel fails, EXC_CHANNEL_OPEN is
+       ; raised. Once the channel is open, this handler is used for
+       ; EXC_FINISH_CHANNEL and EXC_FINISH_PENDING. If the channel was
+       ; opened on the peer's request, the connection's exception
+       ; handler is a parent of the channel's. But that is not true in
+       ; general.
        (e object exception_handler)
 
        ; Resources associated with the channel. This object is also
@@ -124,12 +131,15 @@
        (open_continuation object command_continuation)
 
        ; Queue of channel requests that we expect replies on
-       (pending_requests struct object_queue)))
+       (pending_requests struct object_queue)
+
+       ; Channel requests that we have received, and should reply to
+       ; in the right order
+       (active_requests struct object_queue)))
        
        ; Reply from SSH_MSG_CHANNEL_REQUEST 
        ;; (channel_success method int)
-       ;; (channel_failure method int)))
-*/
+       ;; (channel_failure method int))) */
 
 #define CHANNEL_RECEIVE(s, t, d) \
 ((s)->receive((s), (t), (d)))
@@ -148,6 +158,11 @@
 #define CHANNEL_OPEN_FAILURE(s) \
 ((s)->open_failure((s)))
 
+/* Values used in the in_use array. */
+#define CHANNEL_FREE 0
+#define CHANNEL_RESERVED 1
+#define CHANNEL_IN_USE 2
+
 /* GABA:
    (class
      (name channel_table)
@@ -201,30 +216,22 @@
        ))
 */
 
-
 /* SSH_MSG_GLOBAL_REQUEST */
 
-/* GABA:
-   (class
-     (name global_request_callback)
-     (vars
-       (response method void "int success")
-       (connection object ssh_connection)))
-*/
-
-#define GLOBAL_REQUEST_CALLBACK(c, s) \
-((c)->response((c), (s)))
-
 /* GABA:
    (class
      (name global_request)
      (vars
        (handler method void "struct ssh_connection *connection"
+                            "UINT32 type"
+			    ;; FIXME: Is want-reply really needed?
+                            "int want_reply"
                             "struct simple_buffer *args"
-			    "struct global_request_callback *response")))
+			    "struct command_continuation *c"
+			    "struct exception_handler *e")))
 */
 
-#define GLOBAL_REQUEST(r, c, a, n) ((r)->handler((r), (c), (a), (n)))
+#define GLOBAL_REQUEST(r, c, t, w, a, n, e) ((r)->handler((r), (c), (t), (w), (a), (n), (e)))
 
 /* SSH_MSG_CHANNEL_OPEN */
 
@@ -265,20 +272,15 @@ make_channel_open_exception(UINT32 error_code, const char *msg);
        (handler method void
 		"struct ssh_channel *channel"
 		"struct ssh_connection *connection"
+		"UINT32 type"
 		"int want_reply"
-		"struct simple_buffer *args")))
+		"struct simple_buffer *args"
+		"struct command_continuation *c"
+		"struct exception_handler *e")))
 */
 
-#define CHANNEL_REQUEST(s, c, conn, w, a) \
-((s)->handler((s), (c), (conn), (w), (a)))
-
-/* ;;GABA:
-   (class
-     (name connection_startup)
-     (vars
-       (start method int
-	      "struct ssh_connection *connection")))
-*/
+#define CHANNEL_REQUEST(s, c, conn, t, w, a, n, e) \
+((s)->handler((s), (c), (conn), (t), (w), (a), (n), (e)))
 
 /* #define CONNECTION_START(c, s) ((c)->start((c), (s))) */
 
@@ -288,18 +290,26 @@ struct channel_table *make_channel_table(void);
 int alloc_channel(struct channel_table *table);
 void dealloc_channel(struct channel_table *table, int i);
 
+void
+use_channel(struct ssh_connection *connection,
+	    UINT32 local_channel_number);
+
 void register_channel(struct ssh_connection *connection,
 		      UINT32 local_channel_number,
-		      struct ssh_channel *channel);
+		      struct ssh_channel *channel,
+		      int take_into_use);
 
-struct ssh_channel *lookup_channel(struct channel_table *table, UINT32 i);
+struct ssh_channel *
+lookup_channel(struct channel_table *table, UINT32 i);
+struct ssh_channel *
+lookup_channel_reserved(struct channel_table *table, UINT32 i);
 
 struct abstract_write *make_channel_write(struct ssh_channel *channel);
 struct abstract_write *make_channel_write_extended(struct ssh_channel *channel,
 						   UINT32 type);
 
-struct io_read_callback *make_channel_read_data(struct ssh_channel *channel);
-struct io_read_callback *make_channel_read_stderr(struct ssh_channel *channel);
+struct io_callback *make_channel_read_data(struct ssh_channel *channel);
+struct io_callback *make_channel_read_stderr(struct ssh_channel *channel);
 
 struct lsh_string *format_global_failure(void);
 struct lsh_string *format_global_success(void);
@@ -318,10 +328,16 @@ struct lsh_string *prepare_window_adjust(struct ssh_channel *channel,
 
 void channel_start_receive(struct ssh_channel *channel);
 
+#if 0
 struct lsh_string *prepare_channel_open(struct ssh_connection *connection,
 					int type,
 					struct ssh_channel *channel,
 					const char *format, ...);
+#endif
+struct lsh_string *
+format_channel_open(int type, UINT32 local_channel_number,
+		    struct ssh_channel *channel,
+		    const char *format, ...);
 
 struct lsh_string *format_channel_request(int type,
 					  struct ssh_channel *channel,
