@@ -1,6 +1,6 @@
 /* keyexchange.c
  *
- * $Id: keyexchange.c,v 1.60 2000/01/09 18:32:15 nisse Exp $ */
+ * $Id: keyexchange.c,v 1.61 2000/02/02 22:37:48 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -346,7 +346,7 @@ static struct lsh_string *kex_make_key(struct hash_instance *secret,
 				       struct lsh_string *session_id)
 {
   /* Indexed by the KEX_* values */
-  static /* const */ char *tags = "CDEFAB";
+  static /* const */ UINT8 tags[] = "CDEFAB";
   
   struct lsh_string *key;
   struct hash_instance *hash;
