@@ -1,6 +1,6 @@
 /* tcpforward.c
  *
- * $Id: tcpforward.c,v 1.36 2000/01/26 22:21:47 nisse Exp $
+ * $Id: tcpforward.c,v 1.39 2000/02/06 22:27:23 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -109,7 +109,7 @@ remove_forward(struct object_queue *q, int null_ok,
      (name tcpip_channel)
      (super ssh_channel)
      (vars
-       (socket object io_fd)))
+       (socket object lsh_fd)))
 */
 
 static void
@@ -138,7 +138,7 @@ do_tcpip_send(struct ssh_channel *s,
 {
   CAST(tcpip_channel, self, s);
   
-  self->socket->super.want_read = 1;
+  self->socket->want_read = 1;
 }
 
 static void
@@ -155,10 +155,11 @@ do_tcpip_channel_die(struct ssh_channel *c)
   CAST(tcpip_channel, channel, c);
 
   if (channel->socket)
-    close_fd(&channel->socket->super, 0);
+    close_fd(channel->socket, 0);
 }
 
-struct ssh_channel *make_tcpip_channel(struct io_fd *socket, UINT32 max_window)
+struct ssh_channel *
+make_tcpip_channel(struct lsh_fd *socket, UINT32 max_window)
 {
   NEW(tcpip_channel, self);
   assert(socket);
@@ -343,7 +344,7 @@ make_channel_open_direct_tcpip(struct command *callback)
      (vars
        (connection object ssh_connection)
        (forward object local_port)
-       (c object global_request_callback)))
+       (c object command_continuation)))
 */
 
 /* FIXME: Split off an exception handler */
@@ -367,20 +368,20 @@ do_tcpip_forward_request_continuation(struct command_continuation *c,
       assert(port);
       assert(port == self->forward);
       
-      GLOBAL_REQUEST_CALLBACK(self->c, 0);
+      COMMAND_RETURN(self->c, NULL);
     }
   
   REMEMBER_RESOURCE(self->connection->resources, &fd->super);
 
   self->forward->socket = fd;
 
-  GLOBAL_REQUEST_CALLBACK(self->c, 1);
+  COMMAND_RETURN(self->c, &self->forward->super.super);
 }
 
 static struct command_continuation *
 make_tcpip_forward_request_continuation(struct ssh_connection *connection,
 					struct local_port *forward,
-					struct global_request_callback *c)
+					struct command_continuation *c)
 {
   NEW(tcpip_forward_request_continuation, self);
 
@@ -406,8 +407,11 @@ make_tcpip_forward_request_continuation(struct ssh_connection *connection,
 static void
 do_tcpip_forward_request(struct global_request *s, 
 			 struct ssh_connection *connection,
+			 UINT32 type UNUSED,
+			 int want_reply UNUSED,
 			 struct simple_buffer *args,
-			 struct global_request_callback *c)
+			 struct command_continuation *c,
+			 struct exception_handler *e)
 {
   CAST(tcpip_forward_request, self, s);
   struct lsh_string *bind_host;
@@ -423,15 +427,18 @@ do_tcpip_forward_request(struct global_request *s,
       if (bind_port < 1024)
 	{
 	  werror("Denying forwarding of privileged port %i.\n", bind_port);
-	  GLOBAL_REQUEST_CALLBACK(c, 0);
+	  COMMAND_RETURN(c, NULL);
 	  return;
 	}
 
       if (lookup_forward(&connection->table->local_ports,
 			 bind_host->length, bind_host->data, bind_port))
 	{
+	  static const struct exception again = 
+	    STATIC_EXCEPTION(EXC_GLOBAL_REQUEST, "An already requested tcp-forward requested again");
+
 	  verbose("An already requested tcp-forward requested again\n");
-	  GLOBAL_REQUEST_CALLBACK(c, 0);
+	  EXCEPTION_RAISE(e, &again);
 	  return;
 	}
       
@@ -445,7 +452,9 @@ do_tcpip_forward_request(struct global_request *s,
 		     a,
 		     make_tcpip_forward_request_continuation
 		     (connection, forward, c),
-		     /* FIXME: Use a better exception handler */
+		     /* FIXME: Use a better exception handler.
+		      * We should handle EXC_RESOLVE and EXC_LISTEN, and
+		      * report or ignore i/o-errors. */
 		     &default_exception_handler
 		     /* make_tcpip_forward_request_raise
 			(connection, forward) */ );
@@ -455,7 +464,7 @@ do_tcpip_forward_request(struct global_request *s,
   else
     {
       werror("Incorrectly formatted tcpip-forward request\n");
-      PROTOCOL_ERROR(connection->e, "Invalid tcpip-forward message.");
+      PROTOCOL_ERROR(e, "Invalid tcpip-forward message.");
     }
 }
 
@@ -472,8 +481,11 @@ struct global_request *make_tcpip_forward_request(struct command *callback)
 static void
 do_tcpip_cancel_forward(struct global_request *s UNUSED, 
 			struct ssh_connection *connection,
+			UINT32 type UNUSED,
+			int want_reply UNUSED,
 			struct simple_buffer *args,
-			struct global_request_callback *c)
+			struct command_continuation *c,
+			struct exception_handler *e)
 {
   UINT32 bind_host_length;
   UINT8 *bind_host;
@@ -499,14 +511,16 @@ do_tcpip_cancel_forward(struct global_request *s UNUSED,
 	  close_fd(port->socket, 0);
 	  port->socket = NULL;
 
-	  GLOBAL_REQUEST_CALLBACK(c, 1);
+	  COMMAND_RETURN(c, NULL);
 	  return;
 	}
       else
 	{      
+	  static const struct exception notfound = 
+	    STATIC_EXCEPTION(EXC_GLOBAL_REQUEST, "Could not find tcpip-forward to cancel");
 	  verbose("Could not find tcpip-forward to cancel\n");
 
-	  GLOBAL_REQUEST_CALLBACK(c, 0);
+	  EXCEPTION_RAISE(e, &notfound);
 	  return;
 	}
     }
