@@ -1,6 +1,6 @@
 /* tcpforward.h
  *
- * $Id: tcpforward.h,v 1.11 1999/11/16 22:13:46 nisse Exp $
+ * $Id: tcpforward.h,v 1.12 2000/02/05 17:29:23 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -74,7 +74,7 @@ struct remote_port *
 make_remote_port(struct address_info *listen,
 		 struct command *callback);
 
-struct ssh_channel *make_tcpip_channel(struct io_fd *socket, UINT32 max_window);
+struct ssh_channel *make_tcpip_channel(struct lsh_fd *socket, UINT32 max_window);
 void tcpip_channel_start_io(struct ssh_channel *c);
 
 struct channel_open *
