@@ -1,6 +1,6 @@
 /* tty.h
  *
- * $Id: tty.h,v 1.7 1999/11/16 22:13:48 nisse Exp $ */
+ * $Id: tty.h,v 1.8 2000/02/02 22:44:45 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -32,8 +32,8 @@ int tty_getattr(int fd, struct termios *ios);
 int tty_setattr(int fd, struct termios *ios);
 
 int tty_makeraw(int fd);
-int tty_getwinsize(int fd, int *w, int *h, int *wp, int *hp);
-int tty_setwinsize(int fd, int w, int h, int wp, int hp);
+int tty_getwinsize(int fd, UINT32 *w, UINT32 *h, UINT32 *wp, UINT32 *hp);
+int tty_setwinsize(int fd, UINT32 w, UINT32 h, UINT32 wp, UINT32 hp);
 
 struct lsh_string *tty_encode_term_mode(struct termios *ios);
 int tty_decode_term_mode(struct termios *ios, UINT32 t_len, UINT8 *t_modes);
